/********************************************************************************
 * Copyright (c) 2023 CEA-List
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 ********************************************************************************/

#ifndef __AIDGE_CPU_OPERATOR_ScalingIMPL_FORWARD_KERNEL_H__
#define __AIDGE_CPU_OPERATOR_ScalingIMPL_FORWARD_KERNEL_H__

#include "aidge/utils/Registrar.hpp"

#include "aidge/backend/cpu/operator/ScalingImpl.hpp"

namespace Aidge {
template <class I, class O>
void ScalingImpl_cpu_forward_kernel(const Scaling_Op::Parameters& params,
                                     std::size_t inputLenght,
                                     const void* input_,
                                     void* output_) {

    const I* input = static_cast<const I*>(input_);
    O* output = static_cast<O*>(output_);
    I scalingFactor = static_cast<I>(std::get<0>(params));

    for (std::size_t i = 0; i < inputLenght; ++i) {
        output[i] = input[i] * scalingFactor;
    }
}

namespace {
static Registrar<ScalingImplForward_cpu> registrarScalingImplForward_cpu_Float32(
        {DataType::Float32, DataType::Float32}, Aidge::ScalingImpl_cpu_forward_kernel<float, float>);
static Registrar<ScalingImplForward_cpu> registrarScalingImplForward_cpu_Int32(
        {DataType::Int32, DataType::Int32}, Aidge::ScalingImpl_cpu_forward_kernel<int, int>);
static Registrar<ScalingImplForward_cpu> registrarScalingImplForward_cpu_Float64(
        {DataType::Float64, DataType::Float64}, Aidge::ScalingImpl_cpu_forward_kernel<double, double>);
}  // namespace
}  // namespace Aidge

#endif /* __AIDGE_CPU_OPERATOR_ScalingIMPL_FORWARD_KERNEL_H__ */
